@@ -429,3 +429,61 @@ def git_deploy(git_repo, source_directory, target_domain, target_stage, verbose,
         include_lock_file=include_lock_file,
         working_dir=working_dir,
     )
+
+@main.command(
+    name="show-graph",
+    context_settings=dict(
+        ignore_unknown_options=True,
+    ),
+)
+@click.option('--verbose', '-v', is_flag=True, default=False,
+    help='Verbose printing',
+    show_default=True
+    )
+@click.option('--exclude-external-dependencies', '-x', is_flag=True, default=False,
+    help='Whether or not external dependencies must be ignored',
+    show_default=True
+    )
+@click.option(
+    '--working-dir', '-w', default=None,
+    help='Working directory, when omitted the current directory is used',
+    )
+@click.argument('terragrunt-args', nargs=-1, type=click.UNPROCESSED)
+@click.version_option(version=__version__)
+def show_graph(verbose, exclude_external_dependencies, working_dir, terragrunt_args):
+    """ Shows the dependencies of a project """
+
+    check_latest_version()
+
+    tgwrap = TgWrap(verbose=verbose)
+    tgwrap.show_graph(
+        exclude_external_dependencies=exclude_external_dependencies,
+        working_dir=working_dir,
+        terragrunt_args=terragrunt_args,
+    )
+
+@main.command(
+    name="clean",
+    context_settings=dict(
+        ignore_unknown_options=True,
+    ),
+)
+@click.option('--verbose', '-v', is_flag=True, default=False,
+    help='Verbose printing',
+    show_default=True
+    )
+@click.option(
+    '--working-dir', '-w', default=None,
+    help='Working directory, when omitted the current directory is used',
+    )
+@click.version_option(version=__version__)
+def clean(verbose, working_dir):
+    """ Clean the temporary files of a terragrunt/terraform project """
+
+    check_latest_version()
+
+    tgwrap = TgWrap(verbose=verbose)
+    tgwrap.clean(
+        working_dir=working_dir,
+    )
+
