@@ -33,34 +33,56 @@ class Printer():
 
         self._print_verbose = verbose
 
-    def header(self, msg):
-        msg = msg.strip() if isinstance(msg, str) else msg
-        click.secho('\n' + msg, bold=True, file=sys.stderr)
+    @property
+    def print_verbose(self):
+        return self._print_verbose
 
-    def verbose(self, msg):
+    def line(self):
+        width, _ = os.get_terminal_size()
+        click.secho("-" * width, file=sys.stderr)
+
+    def header(self, msg, print_line_before=False, print_line_after=False):
+        msg = msg.strip() if isinstance(msg, str) else msg
+        self.line() if print_line_before else None
+        click.secho('\n' + msg, bold=True, file=sys.stderr)
+        self.line() if print_line_after else None
+
+    def verbose(self, msg, print_line_before=False, print_line_after=False):
         if self._print_verbose:
             msg = msg.strip() if isinstance(msg, str) else msg
+            self.line() if print_line_before else None
             print(msg, flush=True, file=sys.stderr)
+            self.line() if print_line_after else None
 
-    def normal(self, msg):
+    def normal(self, msg, print_line_before=False, print_line_after=False):
         msg = msg.strip() if isinstance(msg, str) else msg
+        self.line() if print_line_before else None
         print(msg, flush=True, file=sys.stderr)
+        self.line() if print_line_after else None
 
-    def bold(self, msg):
+    def bold(self, msg, print_line_before=False, print_line_after=False):
         msg = msg.strip() if isinstance(msg, str) else msg
+        self.line() if print_line_before else None
         click.secho('\n' + msg, bold=True, file=sys.stderr)
+        self.line() if print_line_after else None
 
-    def warning(self, msg):
+    def warning(self, msg, print_line_before=False, print_line_after=False):
         msg = msg.strip() if isinstance(msg, str) else msg
+        self.line() if print_line_before else None
         click.secho(msg, fg="yellow", bold=True, file=sys.stderr)
+        self.line() if print_line_after else None
 
-    def error(self, msg):
+    def error(self, msg, print_line_before=False, print_line_after=False):
         msg = msg.strip() if isinstance(msg, str) else msg
+        self.line() if print_line_before else None
         click.secho(msg, fg="red", bold=True, file=sys.stderr)
+        self.line() if print_line_after else None
 
-    def success(self, msg):
+    def success(self, msg, print_line_before=False, print_line_after=False):
         msg = msg.strip() if isinstance(msg, str) else msg
+        self.line() if print_line_before else None
         click.secho(msg, fg="green", bold=True, file=sys.stderr)
+        self.line() if print_line_after else None
 
 class TgWrap():
     """
@@ -75,29 +97,29 @@ class TgWrap():
         # Check if the "TERRAGRUNT_SOURCE" environment variable is set
         env_var = "TERRAGRUNT_SOURCE"
         if env_var in os.environ:
-            self.printer.warning(
+            self.printer.bold(
                 f"'{env_var}' environment variable is set with address: '{os.environ[env_var]}'!"
                 )
         else:
-            self.printer.warning(
+            self.printer.bold(
                 f"No '{env_var}' variable is set, so the sources as defined in terragrunt.hcl files will be used as is!"
                 )
 
-    def is_installed(self, program):
+    def _is_installed(self, program):
         """ Checks if a program is installed on the system """
         return shutil.which(program) is not None
 
-    def construct_command(self, command, debug, exclude_external_dependencies,
+    def _construct_command(self, command, debug, exclude_external_dependencies,
         non_interactive=True, no_auto_approve=True, no_lock=True, update_source=False,
         working_dir=None, limit_parallelism=None, terragrunt_args=()):
         """ Constructs the command """
         commands = {
-            'generic': '{base_command} {command} --terragrunt-non-interactive {no_auto_approve} {ignore_deps} {debug_level} {update_source} {working_dir}  {parallelism} {terragrunt_args}',
+            'generic': '{base_command} {command} --terragrunt-non-interactive {no_auto_approve} {ignore_deps} {debug_level} {update_source} {working_dir} {parallelism} {terragrunt_args}',
             'info': '{base_command} terragrunt-info --terragrunt-non-interactive {ignore_deps} {debug_level} {update_source} {working_dir} {terragrunt_args}',
-            'plan': '{base_command} {command} --terragrunt-non-interactive  -out=planfile {ignore_deps} {debug_level} {lock_level} {update_source} {working_dir}  {parallelism} {terragrunt_args}',
-            'apply': '{base_command} {command} {non_interactive} {no_auto_approve} {parallelism} {ignore_deps} {debug_level} {update_source} {working_dir}  {parallelism} {terragrunt_args}',
-            'show': '{base_command} {command} --terragrunt-non-interactive {ignore_deps} {update_source} -json planfile', # no working dir!!!
-            'destroy': '{base_command} {command} --terragrunt-non-interactive --terragrunt-no-auto-approve {ignore_deps} {debug_level} {working_dir} {parallelism} {terragrunt_args}',
+            'plan': '{base_command} {command} --terragrunt-non-interactive  -out=planfile {ignore_deps} {debug_level} {lock_level} {update_source} {working_dir} {parallelism} {terragrunt_args}',
+            'apply': '{base_command} {command} {non_interactive} {no_auto_approve} {parallelism} {ignore_deps} {debug_level} {update_source} {working_dir} {parallelism} {terragrunt_args}',
+            'show': '{base_command} {command} --terragrunt-non-interactive {ignore_deps} {update_source} -json planfile', # no working dir allowed here!!!
+            'destroy': '{base_command} {command} {non_interactive} {no_auto_approve} {ignore_deps} {debug_level} {working_dir} {parallelism} {terragrunt_args}',
         }
 
         lock_stmt         = '-lock=false' if no_lock else ''
@@ -135,18 +157,24 @@ class TgWrap():
 
         return full_command
 
-    def prepare_groups(self, graph, exclude_external_dependencies):
+    def _prepare_groups(self, graph, exclude_external_dependencies, working_dir):
         """ Prepare the list of groups that will be executed """
+
+        working_dir = os.path.abspath(working_dir) if working_dir else os.getcwd()
+        self.printer.verbose(f"Check for working dir: {working_dir}")
+
         groups = []
         for group in nx.topological_generations(graph):
             try:
                 group.remove("\\n") # terragrunt is adding this in some groups for whatever reason
             except ValueError:
                 pass
+
             for idx, directory in enumerate(group):
-                if directory.startswith("/") \
-                    and exclude_external_dependencies \
-                    and directory != os.getcwd():
+                common_path = os.path.commonpath([working_dir, os.path.abspath(directory)])
+                # self.printer.verbose(f'Common path for dir {directory}: {common_path}')
+                if common_path != working_dir \
+                    and exclude_external_dependencies:
                     self.printer.verbose(
                         f"- Remove directory from group as it falls out of scope: {directory}"
                         )
@@ -161,7 +189,7 @@ class TgWrap():
 
         return groups
 
-    def get_di_graph(self, working_dir=None):
+    def _get_di_graph(self, working_dir=None):
         """ Gets the directed graph of terragrunt dependencies, and parse it into a graph object """
         graph = None
         try:
@@ -190,20 +218,25 @@ class TgWrap():
 
         return graph
 
-    def run_di_graph(
+    def _run_di_graph(
         self, command, exclude_external_dependencies, dry_run,
         ask_for_confirmation=False, collect_output_file=None,
         working_dir=None,
         ):
         "Runs the desired command in the directories as defined in the directed graph"
-        graph = self.get_di_graph(working_dir=working_dir)
+
+        graph = self._get_di_graph(working_dir=working_dir)
 
         # first go through the groups and clean up where needed
-        groups = self.prepare_groups(
-            graph=graph, exclude_external_dependencies=exclude_external_dependencies
+        groups = self._prepare_groups(
+            graph=graph,
+            exclude_external_dependencies=exclude_external_dependencies,
+            working_dir=working_dir,
             )
 
-        if ask_for_confirmation or self.printer.verbose:
+        if not groups:
+            self.printer.error('No groups to process, this smells fishy!')
+        elif ask_for_confirmation or self.printer.verbose:
             self.printer.header("The following groups will be processed:")
             for idx, group in enumerate(groups):
                 self.printer.normal(f"\nGroup {idx+1}:")
@@ -224,11 +257,15 @@ class TgWrap():
                 for directory in group:
                     # if we have a specific working dir, and the dir is relative, combine the two
                     if working_dir and not os.path.isabs(directory):
-                        directory = os.path.join(working_dir, directory)
+                        directory = os.path.join(os.path.abspath(working_dir), directory)
+                        self.printer.verbose(f'Executing in directory: {working_dir}')
 
                     self.printer.header(
-                        f'\n\nStart processing directory: {directory}\n\n'
+                        f'\n\nStart processing directory: {directory}\n\n',
+                        print_line_before=True,
+                        print_line_after=True,
                         )
+
                     if dry_run:
                         self.printer.warning(
                             'In dry run mode, no real actions are executed!!'
@@ -243,7 +280,6 @@ class TgWrap():
 
                             with tempfile.NamedTemporaryFile(mode='w+', prefix='tgwrap-', delete=False) as f:
                                 self.printer.verbose(f"Opened temp file for error collection: {f.name}")
-
                                 rc = {'returncode': 0}
                                 rc = subprocess.run(
                                     shlex.split(command),
@@ -259,14 +295,22 @@ class TgWrap():
 
                             if rc.returncode != 0 or 'error' in messages.lower():
                                 raise Exception(
-                                    f'An error situation detected while processing the terragrunt dependencies graph'
+                                    f'An error situation detected while processing the terragrunt dependencies graph in directory {directory}'
                                     )
+                            else:
+                                self.printer.warning(
+                                    f'Directory {directory} processed successfully',
+                                    print_line_before=True,
+                                    print_line_after=True,
+                                )
+
                         except FileNotFoundError:
                             self.printer.warning(f'Directory {directory} not found, continue')
                         except Exception as e:
                             self.printer.error(f"Error occurred:\n{str(e)}")
-                            self.printer.error("Full stack:")
-                            self.printer.normal(messages)
+                            self.printer.error("Full stack:", print_line_before=True)
+                            self.printer.normal(messages, print_line_after=True)
+                            self.printer.normal(f"Directory {directory} failed!")
 
                             stop_processing = True
                             break
@@ -276,6 +320,60 @@ class TgWrap():
             if stop_processing:
                 break
 
+    def _run_sync(
+        self, source_path, target_path, source_stage, target_stage,
+        include_lock_file, dry_run, clean_up, git_source_path=None, source_domain=None,
+        ):
+        """ Run a sync copying files from a source to a target path """
+
+        if not self._is_installed('rsync'):
+            self.printer.error("'rsync' seems not installed. Cannot continue")
+        elif not os.path.exists(source_path):
+            self.printer.error(f"Cannot find {source_path}. Cannot continue.")
+            if source_domain:
+                self.printer.error(
+                    "Please ensure you are in the directory that contains your projects, " + \
+                    "or use --working-dir option"
+                )
+            else:
+                self.printer.error(
+                    "Please ensure you are in the root of your project, or use --working-dir option"
+                )
+        else:
+            self.printer.verbose(f"Copying config: {source_path} => {target_path}")
+
+            try:
+                os.makedirs(target_path)
+            except OSError:
+                # directory already exists
+                pass
+
+            dry_run_stmt = '--dry-run' if dry_run else ''
+            clean_stmt   = '--delete' if clean_up else ''
+            env_file_stmt   = "--exclude='env.hcl'" if source_stage != target_stage else "--include='env.hcl'"
+            lock_file_stmt  = "--include='.terraform.lock.hcl'" if include_lock_file \
+                else "--exclude='.terraform.lock.hcl'"
+
+            cmd = f"rsync -aim {dry_run_stmt} {clean_stmt} " + \
+                f"--include='terragrunt.hcl' {lock_file_stmt} {env_file_stmt} " + \
+                "--exclude='.terragrunt-cache/' --exclude='.terraform/' " + \
+                "--exclude='terragrunt-debug.tfvars.json' --exclude=planfile " + \
+                "--exclude='.DS_Store' " + \
+                f"{source_path} {target_path}"
+
+            cmd = re.sub(' +', ' ', cmd)
+
+            self.printer.header("Will be deploying:", print_line_before=True)
+            self.printer.normal(f"from: {git_source_path if git_source_path else source_path}")
+            self.printer.normal(f"to:   {target_path}")
+            self.printer.verbose(f"Using command:\n$ {cmd}")
+            response = input("\nDo you want to continue? (y/n) ")
+            if response.lower() != "y":
+                sys.exit(1)
+
+            rc = subprocess.run(shlex.split(cmd))
+            self.printer.verbose(rc)
+
     def run(self, command, debug, dry_run, no_lock, update_source,
         auto_approve, working_dir, terragrunt_args):
         """ Executes a terragrunt command on a single module """
@@ -293,7 +391,7 @@ class TgWrap():
                 )
             sys.exit(1)
 
-        cmd = self.construct_command(
+        cmd = self._construct_command(
             command=command,
             debug=debug,
             exclude_external_dependencies=True,
@@ -322,16 +420,15 @@ class TgWrap():
         if terragrunt_args:
             self.printer.verbose(f"- with additional parameters: {' '.join(terragrunt_args)}")
 
-        cmd = self.construct_command(
+        cmd = self._construct_command(
             command=command,
             debug=debug,
             exclude_external_dependencies=True if step_by_step else exclude_external_dependencies,
-            # non_interactive=step_by_step if command in ['apply', 'destroy'] else True,
             non_interactive=auto_approve,
             no_lock=no_lock,
             update_source=update_source,
             no_auto_approve=(not auto_approve),
-            working_dir=working_dir,
+            working_dir=None if step_by_step else working_dir,
             terragrunt_args=terragrunt_args,
             limit_parallelism=limit_parallelism,
         )
@@ -340,7 +437,7 @@ class TgWrap():
             self.printer.verbose(
                 f'This command will be executed for each individual module:\n$ {cmd}'
                 )
-            self.run_di_graph(
+            self._run_di_graph(
                 command=cmd,
                 exclude_external_dependencies=exclude_external_dependencies,
                 dry_run=dry_run,
@@ -405,7 +502,7 @@ class TgWrap():
             self.printer.verbose(f"- with additional parameters: {' '.join(terragrunt_args)}")
 
         # first run a 'show' and write output to file
-        cmd = self.construct_command(
+        cmd = self._construct_command(
             command='show',
             exclude_external_dependencies=True,
             debug=False,
@@ -418,7 +515,7 @@ class TgWrap():
             with tempfile.NamedTemporaryFile(mode='w+', prefix='tgwrap-', delete=False) as f:
                 self.printer.verbose(f"Opened temp file for output collection: {f.name}")
 
-                self.run_di_graph(
+                self._run_di_graph(
                     command=cmd,
                     exclude_external_dependencies=exclude_external_dependencies,
                     dry_run=dry_run,
@@ -500,60 +597,6 @@ class TgWrap():
         if not ts_validation_successful:
             raise Exception("Terrasafe validation failed on one or more modules")
 
-    def run_sync(
-        self, source_path, target_path, source_stage, target_stage,
-        include_lock_file, dry_run, clean_up, git_source_path=None, source_domain=None,
-        ):
-        """ Run a sync copying files from a source to a target path """
-
-        if not self.is_installed('rsync'):
-            self.printer.error("'rsync' seems not installed. Cannot continue")
-        elif not os.path.exists(source_path):
-            self.printer.error(f"Cannot find {source_path}. Cannot continue.")
-            if source_domain:
-                self.printer.error(
-                    "Please ensure you are in the directory that contains your projects, " + \
-                    "or use --working-dir option"
-                )
-            else:
-                self.printer.error(
-                    "Please ensure you are in the root of your project, or use --working-dir option"
-                )
-        else:
-            self.printer.verbose(f"Copying config: {source_path} => {target_path}")
-
-            try:
-                os.makedirs(target_path)
-            except OSError:
-                # directory already exists
-                pass
-
-            dry_run_stmt = '--dry-run' if dry_run else ''
-            clean_stmt   = '--delete' if clean_up else ''
-            env_file_stmt   = "--exclude='env.hcl'" if source_stage != target_stage else "--include='env.hcl'"
-            lock_file_stmt  = "--include='.terraform.lock.hcl'" if include_lock_file \
-                else "--exclude='.terraform.lock.hcl'"
-
-            cmd = f"rsync -aim {dry_run_stmt} {clean_stmt} " + \
-                f"--include='terragrunt.hcl' {lock_file_stmt} {env_file_stmt} " + \
-                "--exclude='.terragrunt-cache/' --exclude='.terraform/' " + \
-                "--exclude='terragrunt-debug.tfvars.json' --exclude=planfile " + \
-                "--exclude='.DS_Store' " + \
-                f"{source_path} {target_path}"
-
-            cmd = re.sub(' +', ' ', cmd)
-
-            self.printer.header("Will be deploying:")
-            self.printer.normal(f"from: {git_source_path if git_source_path else source_path}")
-            self.printer.normal(f"to:   {target_path}")
-            self.printer.verbose(f"Using command:\n$ {cmd}")
-            response = input("\nDo you want to continue? (y/n) ")
-            if response.lower() != "y":
-                sys.exit(1)
-
-            rc = subprocess.run(shlex.split(cmd))
-            self.printer.verbose(rc)
-
     def deploy(
         self, source_stage, target_stage, source_domain, target_domain, module,
         dry_run, clean_up, include_lock_file, working_dir, 
@@ -575,7 +618,7 @@ class TgWrap():
         source_path = os.path.join(working_dir, source_domain, source_stage, module, '')
         target_path = os.path.join(working_dir, target_domain, target_stage, module, '')
 
-        self.run_sync(
+        self._run_sync(
             source_path=source_path,
             target_path=target_path,
             source_domain=source_domain,
@@ -591,6 +634,7 @@ class TgWrap():
         dry_run, include_lock_file, working_dir, 
         ):
         """ Deploys the terragrunt config files from a git repository """
+
         DEFAULT_VERSION='latest'
 
         # first try to determine the desired version, this should be coded in
@@ -619,6 +663,8 @@ class TgWrap():
             rc = subprocess.run(
                 shlex.split(cmd),
                 check=True,
+                stdout=sys.stdout if self.printer.print_verbose else subprocess.DEVNULL,
+                stderr=sys.stderr if self.printer.print_verbose else subprocess.DEVNULL,
                 )
             self.printer.verbose(rc)
 
@@ -629,6 +675,8 @@ class TgWrap():
                     shlex.split(cmd),
                     cwd=temp_dir,
                     check=True,
+                    stdout=sys.stdout if self.printer.print_verbose else subprocess.DEVNULL,
+                    stderr=sys.stderr if self.printer.print_verbose else subprocess.DEVNULL,
                     )
                 self.printer.verbose(rc)
 
@@ -641,7 +689,8 @@ class TgWrap():
             source_type = reversed_path[1]
 
             if target_stage == 'global' and source_stage != target_stage:
-                self.printer.error(f'You are trying to copy source stage {source_stage} to a globa stage!')
+                self.printer.error(f'You are trying to copy source stage {source_stage} to a global stage!')
+                self.printer.error('This is not correct, exit.')
                 sys.exit(1)
 
             reversed_path = list(reversed(working_dir.split('/')))
@@ -657,7 +706,7 @@ class TgWrap():
             source_path = os.path.join(temp_dir, source_directory, '')
             target_path = os.path.join(working_dir, target_domain, target_stage, '')
 
-            self.run_sync(
+            self._run_sync(
                 source_path=source_path,
                 git_source_path=f'{git_repo}//{source_directory}?ref={version}',
                 target_path=target_path,
@@ -668,6 +717,43 @@ class TgWrap():
                 clean_up=False,
             )
 
-
         finally:
-            shutil.rmtree(temp_dir)
\ No newline at end of file
+            shutil.rmtree(temp_dir)
+
+    def show_graph(self, exclude_external_dependencies, working_dir, terragrunt_args):
+        """ Shows the dependencies of a project """
+
+        self.printer.verbose(f"Attempting to show dependencies")
+        if terragrunt_args:
+            self.printer.verbose(f"- with additional parameters: {' '.join(terragrunt_args)}")
+
+        "Runs the desired command in the directories as defined in the directed graph"
+        graph = self._get_di_graph(working_dir=working_dir)
+
+        # first go through the groups and clean up where needed
+        groups = self._prepare_groups(
+            graph=graph,
+            exclude_external_dependencies=exclude_external_dependencies,
+            working_dir=working_dir,
+            )
+
+        self.printer.header("The following groups will be processed:")
+        for idx, group in enumerate(groups):
+            self.printer.normal(f"\nGroup {idx+1}:")
+            for directory in group:
+                self.printer.normal(f"- {directory}")
+
+    def clean(self, working_dir):
+        """ Clean the temporary files of a terragrunt/terraform project """
+
+        cmd = 'find . -name ".terragrunt-cache" -type d -exec rm -rf {} \; ; find . -name ".terraform" -type d -exec rm -rf {} \;'
+        rc = subprocess.run(
+            cmd,
+            shell=True,
+            check=True,
+            stdout=sys.stdout if self.printer.print_verbose else subprocess.DEVNULL,
+            stderr=sys.stderr if self.printer.print_verbose else subprocess.DEVNULL,
+            cwd=working_dir if working_dir else None,
+            )
+        self.printer.verbose(rc)
+
